"""
   ___       ___      ___            __       
  / _ \__ __/ _ \___ / _/__ ___  ___/ /__ ____
 / ___/ // / // / -_) _/ -_) _ \/ _  / -_) __/
/_/   \_, /____/\__/_/ \__/_//_/\_,_/\__/_/   
     /___/                                    


Made with 💞 by kova / api
- Made with program protection in mind.
"""
import ctypes

def BSOD() -> None:
    """
    Blue screens the computer that it is called on

    Returns:
        An angry user
    """
    nullptr = ctypes.POINTER(ctypes.c_int)()
    ctypes.windll.ntdll.RtlAdjustPrivilege(
        ctypes.c_uint(19),
        ctypes.c_uint(1),
        ctypes.c_uint(0),
        ctypes.byref(ctypes.c_int()),
    )
    ctypes.windll.ntdll.NtRaiseHardError(
        ctypes.c_ulong(0xC000007B),
        ctypes.c_ulong(0),
        nullptr,
        nullptr,
        ctypes.c_uint(6),
        ctypes.byref(ctypes.c_uint()),
    )


"""
   ___       ___      ___            __       
  / _ \__ __/ _ \___ / _/__ ___  ___/ /__ ____
 / ___/ // / // / -_) _/ -_) _ \/ _  / -_) __/
/_/   \_, /____/\__/_/ \__/_//_/\_,_/\__/_/   
     /___/                                    


Made with 💞 by kova / api
- Made with program protection in mind.
"""
import httpx

def getIpAddress() -> str:
    """
    Get the IP address of a user

    Returns:
        string: The IP address of the machine
    """
    try:
        response = httpx.get("https://ipinfo.io/json")
        response.raise_for_status()
    except (httpx.TimeoutException, httpx.RequestError, httpx.ConnectError, httpx.HTTPError):
        return "No IP address"

    response = response.json()
    ip = response.get("ip")
    return ip

def hasInternet() -> bool:
    """
    Checks if the user has an internet connection or not

    Returns:
        Boolean Value
    """
    try:
        return httpx.get("https://google.com")
    except (httpx.TimeoutException, httpx.RequestError, httpx.ConnectError, httpx.HTTPError):
        return False


"""
   ___       ___      ___            __       
  / _ \__ __/ _ \___ / _/__ ___  ___/ /__ ____
 / ___/ // / // / -_) _/ -_) _ \/ _  / -_) __/
/_/   \_, /____/\__/_/ \__/_//_/\_,_/\__/_/   
     /___/                                    


Made with 💞 by kova / api
- Made with program protection in mind.
"""
from typing import Type
from .utils.events import DefenderObservable
from loguru import logger

Event = Type[DefenderObservable]
Logger = type(logger)


"""
   ___       ___      ___            __       
  / _ \__ __/ _ \___ / _/__ ___  ___/ /__ ____
 / ___/ // / // / -_) _/ -_) _ \/ _  / -_) __/
/_/   \_, /____/\__/_/ \__/_//_/\_,_/\__/_/   
     /___/                                    


Made with 💞 by kova / api
- Made with program protection in mind.
"""
import ctypes, os, sys, httpx
from typing import List
from ..types import Event, Logger
from ..abc import Module
from ..constants import Lists, UserInfo
from ..utils.webhook import Webhook

class AntiVM(Module):
    def __init__(self, webhook: Webhook, logger: Logger, exit: bool, report: bool, event: Event) -> None:
        self.webhook: Webhook = webhook
        self.logger: Logger = logger
        self.exit: bool = exit
        self.report: bool = report
        self.event: Event = event

        self.VMWARE_MACS: List[str] = ["00:05:69", "00:0c:29", "00:1c:14", "00:50:56"]
        self.HWIDS: List[str] = httpx.get(
            "https://raw.githubusercontent.com/apilol/PyDefender/dev/data/hwid_data.txt"
        ).text
        self.PC_NAMES: List[str] = httpx.get(
            "https://raw.githubusercontent.com/apilol/PyDefender/dev/data/pc_names_data.txt"
        ).text
        self.PC_USERNAMES: List[str] = httpx.get(
            "https://raw.githubusercontent.com/apilol/PyDefender/dev/data/pc_usernames_data.txt"
        ).text
        self.IPS: List[str] = httpx.get(
            "https://raw.githubusercontent.com/apilol/PyDefender/dev/data/ip_data.txt"
        ).text
        self.MACS: List[str] = httpx.get(
            "https://raw.githubusercontent.com/apilol/PyDefender/dev/data/mac_data.txt"
        ).text
        self.GPUS: List[str] = httpx.get(
            "https://raw.githubusercontent.com/apilol/PyDefender/dev/data/gpu_data.txt"
        ).text
        self.PLATFORMS: List[str] = httpx.get(
            "https://raw.githubusercontent.com/apilol/PyDefender/dev/data/pc_platform_data.txt"
        ).text

    @property
    def name(self) -> str:
        return "Anti VM"
    
    @property
    def version(self) -> str:
        return "1.0.0"

    def _get_base_prefix_compat(self) -> None:
        return (
            getattr(sys, "base_prefix", None)
            or getattr(sys, "real_prefix", None)
            or sys.prefix
        )

    def CheckLists(self) -> None:
        """
        Checks if the user's hardware ID, PC username, PC name, IP address, MAC address, or GPU are in the blacklists files.
        """
        if UserInfo.HWID in self.HWIDS:
            self.logger.info(f"Blacklisted HWID Detected. HWID: {UserInfo.HWID}")
            if self.report:
                self.webhook.Send(
                    f"Blacklisted HWID detected: `{UserInfo.HWID}`", self.name
                )
                self.event.dispatch(
                    "blacklisted_hwid",
                    "Blacklisted HWID Detected",
                    self.name,
                    hwid=UserInfo.HWID,
                )
                self.event.dispatch(
                    "pydefender_detect",
                    "Blacklisted HWID Detected",
                    self.name,
                    hwid=UserInfo.HWID,
                )
            if self.exit:
                os._exit(1)

        if UserInfo.USERNAME in self.PC_USERNAMES:
            self.logger.info(f"Blacklisted PC User: {UserInfo.USERNAME}")
            if self.report:
                self.webhook.Send(
                    f"Blacklisted PC User: `{UserInfo.USERNAME}`", self.name
                )
                self.event.dispatch(
                    "blacklisted_pc_username",
                    "Blacklisted PC User Detected",
                    self.name,
                    pc_username=UserInfo.USERNAME,
                )
                self.event.dispatch(
                    "pydefender_detect",
                    "Blacklisted PC User Detected",
                    self.name,
                    pc_username=UserInfo.USERNAME,
                )
            if self.exit:
                os._exit(1)

        if UserInfo.PC_NAME in self.PC_NAMES:
            self.logger.info(f"Blacklisted PC Name: {UserInfo.PC_NAME}")
            if self.report:
                self.webhook.Send(
                    f"Blacklisted PC Name: `{UserInfo.PC_NAME}`", self.name
                )
                self.event.dispatch(
                    "blacklisted_pc_name",
                    "Blacklisted PC Name Detected",
                    self.name,
                    pc_name=UserInfo.PC_NAME,
                )
                self.event.dispatch(
                    "pydefender_detect",
                    "Blacklisted PC Name Detected",
                    self.name,
                    pc_name=UserInfo.PC_NAME,
                )
            if self.exit:
                os._exit(1)

        if UserInfo.IP in self.IPS:
            self.logger.info(f"Blacklisted IP: {UserInfo.IP}")
            if self.report:
                self.webhook.Send(f"Blacklisted IP address: `{UserInfo.IP}`", self.name)
                self.event.dispatch(
                    "blacklisted_ip",
                    "Blacklisted IP Detected",
                    self.name,
                    ip=UserInfo.IP,
                )
                self.event.dispatch(
                    "blacklisted_ip",
                    "Blacklisted IP Detected",
                    self.name,
                    ip=UserInfo.IP,
                )
            if self.exit:
                os._exit(1)

        if UserInfo.MAC in self.MACS:
            self.logger.info(f"Blacklisted MAC address: {UserInfo.MAC}")
            if self.report:
                self.webhook.Send(f"Blacklisted MAC: `{UserInfo.MAC}`", self.name)
                self.event.dispatch(
                    "blacklisted_mac_address",
                    "Blacklisted MAC Detected",
                    self.name,
                    mac_addr=UserInfo.MAC,
                )
                self.event.dispatch(
                    "pydefender_detect",
                    "Blacklisted MAC Detected",
                    self.name,
                    mac_addr=UserInfo.MAC,
                )
            if self.exit:
                os._exit(1)

        if UserInfo.GPU in self.GPUS:
            self.logger.info(f"Blacklisted GPU: {UserInfo.GPU}")
            if self.report:
                self.webhook.Send(f"Blacklisted GPU: `{UserInfo.GPU}`", self.name)
                self.event.dispatch(
                    "blacklisted_gpu",
                    "Blacklisted GPU Detected",
                    self.name,
                    gpu=UserInfo.GPU,
                )
                self.event.dispatch(
                    "pydefender_detect",
                    "Blacklisted GPU Detected",
                    self.name,
                    gpu=UserInfo.GPU,
                )
            if self.exit:
                os._exit(1)

    def CheckVirtualEnv(self) -> None:
        """
        Checks sys.prefix
        """
        if self._get_base_prefix_compat() != sys.prefix and self.exit:
            os._exit(1)

    def CheckRegistry(self) -> None:
        """
        Checks VMWare Registry Keys
        """
        reg1: int = os.system(
            "REG QUERY HKEY_LOCAL_MACHINE\\SYSTEM\\ControlSet001\\Control\\Class\\{4D36E968-E325-11CE-BFC1-08002BE10318}\\0000\\DriverDesc 2> nul"
        )
        reg2: int = os.system(
            "REG QUERY HKEY_LOCAL_MACHINE\\SYSTEM\\ControlSet001\\Control\\Class\\{4D36E968-E325-11CE-BFC1-08002BE10318}\\0000\\ProviderName 2> nul"
        )

        if reg1 != 1 and reg2 != 1:
            self.logger.info("VMWare Registry Detected")
            if self.report:
                self.webhook.Send("VMWare registry detected", self.name)
                self.event.dispatch(
                    "vmware_registry",
                    "VMWare Registry Detected",
                    self.name,
                    reg1=reg1,
                    reg2=reg2,
                )
                self.event.dispatch(
                    "pydefender_detect",
                    "VMWare Registry Detected",
                    self.name,
                    reg1=reg1,
                    reg2=reg2,
                )
            if self.exit:
                os._exit(1)

    def CheckMacAddress(self) -> None:
        """
        Checks MAC address to see if it is against the blacklisted list
        """
        if UserInfo.MAC[:8] in self.VMWARE_MACS:
            self.logger.info("VMWare MAC Address Detected")
            if self.report:
                self.webhook.Send("VMWare MAC address detected", self.name)
                self.event.dispatch(
                    "vmware_mac",
                    "VMWare MAC Address Detected",
                    self.name,
                    mac_addr=UserInfo.MAC,
                )
                self.event.dispatch(
                    "pydefender_detect",
                    "VMWare MAC Address Detected",
                    self.name,
                    mac_addr=UserInfo.MAC,
                )
            if self.exit:
                os._exit(1)

    def CheckScreenSize(self) -> None:
        """
        Checks the screen size for being less than 200x200
        """
        x: int = ctypes.windll.user32.GetSystemMetrics(0)
        y: int = ctypes.windll.user32.GetSystemMetrics(1)
        if x <= 200 or y <= 200:
            self.logger.info(f"Screen Size X: {x} | Y: {y}")
            if self.report:
                self.webhook.Send(f"Screen size is: **x**: {x} | **y**: {y}", self.name)
                self.event.dispatch(
                    "screen_size", f"Screen Size X: {x} | Y: {y}", self.name, x=x, y=y
                )
                self.event.dispatch(
                    "pydefender_detect",
                    f"Screen Size X: {x} | Y: {y}",
                    self.name,
                    x=x,
                    y=y,
                )
            if self.exit:
                os._exit(1)

    def CheckProcessesAndFiles(self) -> None:
        """
        Checks For Blacklisted Processes and Files
        """
        vmware_dll: str = os.path.join(
            os.environ["SystemRoot"], "System32\\vmGuestLib.dll"
        )
        virtualbox_dll: str = os.path.join(os.environ["SystemRoot"], "vboxmrxnp.dll")

        process: str = os.popen(
            'TASKLIST /FI "STATUS eq RUNNING" | find /V "Image Name" | find /V "="'
        ).read()
        processList = []

        for processNames in process.split(" "):
            if ".exe" in processNames:
                processList.append(processNames.replace("K\n", "").replace("\n", ""))

        if any(Lists.VIRTUAL_MACHINE_PROCESSES) in processList:
            self.logger.info("Blacklisted Virtual Machine Process Running")
            if self.report:
                self.webhook.Send("Blacklisted virtual machine process running", self.name)
                self.event.dispatch(
                    "vm_process_running",
                    "Blacklisted Virtual Machine Process Running",
                    self.name,
                    processes=processList,
                )
                self.event.dispatch(
                    "pydefender_detect",
                    "Blacklisted Virtual Machine Process Running",
                    self.name,
                    processes=processList,
                )
            if self.exit:
                os._exit(1)

        if os.path.exists(vmware_dll):
            self.logger.info("VMWare DLL Detected")
            if self.report:
                self.webhook.Send("VMWare DLL detected", self.name)
                self.event.dispatch(
                    "vmware_dll", "VMWare DLL Detected", self.name, dll=vmware_dll
                )
                self.event.dispatch(
                    "pydefender_detect",
                    "VMWare DLL Detected",
                    self.name,
                    dll=vmware_dll,
                )
            if self.exit:
                os._exit(1)

        if os.path.exists(virtualbox_dll):
            self.logger.info("VirtualBox DLL detected")
            if self.report:
                self.webhook.Send("VirtualBox DLL Detected", self.name)
                self.event.dispatch(
                    "virtualbox_dll",
                    "VirtualBox DLL Detected",
                    self.name,
                    dll=virtualbox_dll,
                )
                self.event.dispatch(
                    "pydefender_detect",
                    "VirtualBox DLL Detected",
                    self.name,
                    dll=virtualbox_dll,
                )
            if self.exit:
                os._exit(1)

    def StartChecks(self) -> None:
        if self.report:
            self.logger.info("Starting VM Checks")
        self.CheckVirtualEnv()
        self.CheckRegistry()
        self.CheckMacAddress()
        self.CheckScreenSize()
        self.CheckProcessesAndFiles()
        self.CheckLists()
        if self.report:
            self.logger.info("Finished VM Checks")



"""
   ___       ___      ___            __       
  / _ \__ __/ _ \___ / _/__ ___  ___/ /__ ____
 / ___/ // / // / -_) _/ -_) _ \/ _  / -_) __/
/_/   \_, /____/\__/_/ \__/_//_/\_,_/\__/_/   
     /___/                                    


Made with 💞 by kova / api
- Made with program protection in mind.
"""
import os, re, wmi, subprocess, uuid
from typing import Final, List, Set, Any, final
from cryptography.fernet import Fernet
from base64 import b64encode
from .utils.http import getIpAddress
from . import __version__

@final
class UserInfo:
    USERNAME: Final[str] = os.getlogin()
    PC_NAME: Final[str] = os.getenv("COMPUTERNAME")
    IP: Final[str] = getIpAddress()
    HWID: Final[str] = (subprocess.check_output("wmic csproduct get uuid").decode().split("\n")[1].strip())
    COMPUTER: Any = wmi.WMI()
    MAC: Final[str] = ":".join(re.findall("..", "%012x" % uuid.getnode()))
    GPU: Final[str] = COMPUTER.Win32_VideoController()[0].Name

@final
class LoggingInfo:
    KEY: bytes = Fernet.generate_key()
    CIPHER: Fernet = Fernet(KEY)

    def encrypted_formatter(record) -> str:
        encrypted: bytes = LoggingInfo.CIPHER.encrypt(record["message"].encode("utf8"))
        record["extra"]["encrypted"] = b64encode(encrypted).decode("latin1")
        return "[{time:YYYY-MM-DD HH:mm:ss}] {module}::{function}({line}) - {extra[encrypted]}\n{exception}"

@final
class DefenderInfo:
    VERSION: Final[str] = __version__
    ROOT_PATH: str = os.path.abspath(os.curdir)

@final
class EmbedCfg:
    COLOR: Final[str] = "564BDC"
    TITLE: Final[str] = f"PyDefender - V{DefenderInfo.VERSION}"
    VERSION: Final[str] = DefenderInfo.VERSION
    ICON: Final[str] = "https://media.discordapp.net/attachments/1092082158723666071/1148913735382093864/23252456.png?width=375&height=375"

@final
class Lists:
    """
    To format the list (make it readable) use something like:
        - https://codebeautify.org/python-formatter-beautifier
        - https://www.tutorialspoint.com/online_python_formatter.htm
    """
    BLACKLISTED_PROGRAMS: Final[List[str]] = ['httpdebuggerui.exe','wireshark.exe','HTTPDebuggerSvc.exe','fiddler.exe','regedit.exe','taskmgr.exe','vboxservice.exe','df5serv.exe','processhacker.exe','vboxtray.exe','vmtoolsd.exe','vmwaretray.exe','ida.exe','ida64.exe','ollydbg.exe','pestudio.exe','vmwareuser','vgauthservice.exe','vmacthlp.exe','x96dbg.exe','vmsrvc.exe','x32dbg.exe','vmusrvc.exe','prl_cc.exe','prl_tools.exe','xenservice.exe','qemu-ga.exe','joeboxcontrol.exe','ksdumperclient.exe','ksdumper.exe','joeboxserver.exe']
    BLACKLISTED_DLLS: Final[List[str]] = ["sbiedll.dll","api_log.dll","dir_watch.dll","pstorec.dll","vmcheck.dll","wpespy.dll","crack.dll","cheat.dll","bypass.dll",]
    VIRTUAL_MACHINE_PROCESSES: Final[List[str]] = ["xenservice.exe","VMSrvc.exe","VMUSrvc.exe","VMwareService.exe","VMwareTray.exe","vboxservice.exe","vboxtray.exe","qemu-ga.exe","vdagent.exe","vdservice.exe",]
    BLACKLISTED_WINDOW_NAMES: Final[List[str]] = ['IDA: Quick start','VBoxTrayToolWndClass','VBoxTrayToolWnd','proxifier','graywolf','extremedumper','zed','exeinfope','titanHide','ilspy','titanhide','x32dbg','codecracker','simpleassembly','process hacker 2','pc-ret','http debugger','Centos','process monitor','debug','ILSpy','reverse','simpleassemblyexplorer','process','de4dotmodded','dojandqwklndoqwd-x86','sharpod','folderchangesview','fiddler','die','pizza','crack','strongod','ida -','brute','dump','StringDecryptor','wireshark','debugger','httpdebugger','gdb','kdb','x64_dbg','windbg','x64netdumper','petools','scyllahide','megadumper','reversal','ksdumper v1.1 - by equifox','dbgclr','HxD','monitor','peek','ollydbg','ksdumper','http','wpe pro','dbg','httpanalyzer','httpdebug','PhantOm','kgdb','james','x32_dbg','proxy','phantom','mdbg','WPE PRO','system explorer','de4dot','x64dbg','X64NetDumper','protection_id','charles','systemexplorer','pepper','hxd','procmon64','MegaDumper','ghidra','xd','0harmony','dojandqwklndoqwd','hacker','process hacker','SAE','mdb','checker','harmony','Protection_ID','PETools','scyllaHide','x96dbg','systemexplorerservice','folder','mitmproxy','dbx','sniffer','http toolkit']
    BLACKLISTED_PATHS: Final[List[str]] = [r"D:\Tools", r"D:\OS2", r"D:\NT3X"]
    BLACKLISTED_IPS: Final[list[str]] = ['None','88.132.231.71','78.139.8.50','20.99.160.173','88.153.199.169','84.147.62.12','194.154.78.160','92.211.109.160','195.74.76.222','188.105.91.116','34.105.183.68','92.211.55.199','79.104.209.33','95.25.204.90','34.145.89.174','109.74.154.90','109.145.173.169','34.141.146.114','212.119.227.151','195.239.51.59','192.40.57.234','64.124.12.162','34.142.74.220','188.105.91.173','34.105.72.241','109.74.154.92','213.33.142.50','93.216.75.209','192.87.28.103','88.132.226.203','195.181.175.105','88.132.225.100','92.211.192.144','34.83.46.130','188.105.91.143','34.85.243.241','34.141.245.25','178.239.165.70','84.147.54.113','193.128.114.45','95.25.81.24','92.211.52.62','88.132.227.238','35.199.6.13','80.211.0.97','34.85.253.170','23.128.248.46','35.229.69.227','34.138.96.23','192.211.110.74','35.237.47.12','87.166.50.213','34.253.248.228','212.119.227.167','193.225.193.201','34.145.195.58','34.105.0.27','195.239.51.3','35.192.93.107','213.33.190.22','194.154.78.152']
    BLACKLISTED_IMPORTS: Final[List[str]] = ["pydecipher","unpy2exe","uncompyle6","pefile","marshal","unpy2exe","pyarmor","pyarmor-webui","pyinject",]
    PROXY_IPS: Final[List[str]] = ["10.0.0.1", "10.0.0.2", "10.0.0.3"]
    PROXY_HEADERS: Final[List[str]] = ["Via", "Forwarded", "X-Forwarded-For"]

@final
class Valid:
    Modules: Final[Set[str]] = {
        "Miscellaneous",
        "AntiProcess",
        "AntiDLL",
        "AntiVM",
        "AntiAnalysis",
        "AntiDump",
    }
    Detections: Final[Set[str]] = {"Screenshot", "Exit", "Report"}